@@ -7,9 +7,11 @@ from simpy.resources.container import (ContainerAmount, ContainerGet,
 from simpy.resources.resource import PriorityRequest, Release, Request
 from simpy.resources.store import FilterStoreGet, StoreGet, StorePut
 
+from .events import (ContainerSetCapacity, ContainerSetLevel,
+                     ResourceSetCapacity, ResourceSetUtilization,
+                     StoreSetCapacity, StoreSetContent)
+
 from .core import VisualComponent, VisualEnvironment
-from .visualutil import (ContainerVisualUtil, ResourceVisualUtil,
-                         StoreVisualUtil)
 from .primitives import ComponentType, ErrorText
 
 
@@ -28,7 +30,8 @@ class VisualProcess(VisualComponent):
         0x0000FF is blue.
         If the whole image is white, tinting it will change the color of the
         image. If the image is black, tinting it will have no effect.
-        If no tint should be applied, set it to 0xFFFFFF.
+        If no tint should be applied, set it to 0xFFFFFF, which is the default
+        value.
     """
 
     def __init__(
@@ -36,7 +39,7 @@ class VisualProcess(VisualComponent):
             env: VisualEnvironment,
             id: str,
             visual: str,
-            tint: int):
+            tint: int = 0xFFFFFF):
         super().__init__(env, id, ComponentType.PROCESS, visual, tint)
 
 
@@ -56,7 +59,8 @@ class VisualResource(VisualComponent, Resource):
         0x0000FF is blue.
         If the whole image is white, tinting it will change the color of the
         image. If the image is black, tinting it will have no effect.
-        If no tint should be applied, set it to 0xFFFFFF.
+        If no tint should be applied, set it to 0xFFFFFF, which is the default
+        value.
     :raises ValueError: If the capacity is not a positive integer.
     :raises TypeError: If the capacity is not a integer.
     """
@@ -67,25 +71,45 @@ class VisualResource(VisualComponent, Resource):
             id: str,
             capacity: int,
             visual: str,
-            tint: int):
+            tint: int = 0xFFFFFF):
         if not isinstance(capacity, int):
             raise TypeError(ErrorText.CAPACITY_MUST_BE_POSITIVE_INT)
         if capacity <= 0:
             raise ValueError(ErrorText.CAPACITY_MUST_BE_POSITIVE_INT)
-        VisualComponent.__init__(
-            self, env, id, ComponentType.RESOURCE, visual, tint)
+        super().__init__(env, id, ComponentType.RESOURCE, visual, tint)
         Resource.__init__(self, env, capacity)
-        ResourceVisualUtil.set_capacity(self, capacity)
-        ResourceVisualUtil.set_utilization(self, 0)
+        self.__update_capacity()
+        self.__update_utilization()
+
+    @property
+    def capacity(self) -> ContainerAmount:
+        return self._capacity
+
+    @capacity.setter
+    def capacity(self, capacity: ContainerAmount):
+        self._capacity = capacity
+        self.__update_capacity()
+
+    def __update_capacity(self):
+        self.visualization_manager.add_event(
+            ResourceSetCapacity(self.id, self.env.now, self.capacity)
+        )
+
+    def __update_utilization(self, _=None):
+        self.visualization_manager.add_event(
+            ResourceSetUtilization(
+                self.id, self.env.now, self.count))
 
     def request(self) -> Request:
         req = super().request()
-        ResourceVisualUtil.set_utilization(self, self.count)
+        self.__update_utilization()
+        req.callbacks.append(self.__update_utilization)
         return req
 
     def release(self, request: Request) -> Release:
         rel = super().release(request)
-        ResourceVisualUtil.set_utilization(self, self.count)
+        self.__update_utilization()
+        rel.callbacks.append(self.__update_utilization)
         return rel
 
 
@@ -105,7 +129,8 @@ class VisualPreemptiveResource(VisualComponent, PreemptiveResource):
         0x0000FF is blue.
         If the whole image is white, tinting it will change the color of the
         image. If the image is black, tinting it will have no effect.
-        If no tint should be applied, set it to 0xFFFFFF.
+        If no tint should be applied, set it to 0xFFFFFF, which is the default
+        value.
     :raises TypeError: If the capacity is not a integer.
     :raises ValueError: If the capacity is not a positive integer.
     """
@@ -116,28 +141,48 @@ class VisualPreemptiveResource(VisualComponent, PreemptiveResource):
             id: str,
             capacity: int,
             visual: str,
-            tint: int):
+            tint: int = 0xFFFFFF):
         if not isinstance(capacity, int):
             raise TypeError(ErrorText.CAPACITY_MUST_BE_POSITIVE_INT)
         if capacity <= 0:
             raise ValueError(ErrorText.CAPACITY_MUST_BE_POSITIVE_INT)
-        VisualComponent.__init__(
-            self, env, id, ComponentType.RESOURCE, visual, tint)
+        super().__init__(env, id, ComponentType.RESOURCE, visual, tint)
         PreemptiveResource.__init__(self, env, capacity)
-        ResourceVisualUtil.set_capacity(self, capacity)
-        ResourceVisualUtil.set_utilization(self, 0)
+        self.__update_capacity()
+        self.__update_utilization()
+
+    @property
+    def capacity(self) -> ContainerAmount:
+        return self._capacity
+
+    @capacity.setter
+    def capacity(self, capacity: ContainerAmount):
+        self._capacity = capacity
+        self.__update_capacity()
+
+    def __update_capacity(self):
+        self.visualization_manager.add_event(
+            ResourceSetCapacity(self.id, self.env.now, self.capacity)
+        )
+
+    def __update_utilization(self, _=None):
+        self.visualization_manager.add_event(
+            ResourceSetUtilization(
+                self.id, self.env.now, self.count))
 
     def request(
             self,
             priority: int = 0,
             preempt: bool = True) -> PriorityRequest:
         req = super().request(priority, preempt)
-        ResourceVisualUtil.set_utilization(self, self.count)
+        self.__update_utilization()
+        req.callbacks.append(self.__update_utilization)
         return req
 
     def release(self, request: PriorityRequest) -> Release:
         rel = super().release(request)
-        ResourceVisualUtil.set_utilization(self, self.count)
+        self.__update_utilization()
+        rel.callbacks.append(self.__update_utilization)
         return rel
 
 
@@ -157,7 +202,8 @@ class VisualPriorityResource(VisualComponent, PriorityResource):
         0x0000FF is blue.
         If the whole image is white, tinting it will change the color of the
         image. If the image is black, tinting it will have no effect.
-        If no tint should be applied, set it to 0xFFFFFF.
+        If no tint should be applied, set it to 0xFFFFFF, which is the default
+        value.
     :raises TypeError: If the capacity is not a integer.
     :raises ValueError: If the capacity is not a positive integer.
     """
@@ -168,28 +214,48 @@ class VisualPriorityResource(VisualComponent, PriorityResource):
             id: str,
             capacity: int,
             visual: str,
-            tint: int):
+            tint: int = 0xFFFFFF):
         if not isinstance(capacity, int):
             raise TypeError(ErrorText.CAPACITY_MUST_BE_POSITIVE_INT)
         if capacity <= 0:
             raise ValueError(ErrorText.CAPACITY_MUST_BE_POSITIVE_INT)
-        VisualComponent.__init__(self, env, id, ComponentType.RESOURCE,
-                                 visual, tint)
+        super().__init__(env, id, ComponentType.RESOURCE, visual, tint)
         PriorityResource.__init__(self, env, capacity)
-        ResourceVisualUtil.set_capacity(self, capacity)
-        ResourceVisualUtil.set_utilization(self, 0)
+        self.__update_capacity()
+        self.__update_utilization()
+
+    @property
+    def capacity(self) -> ContainerAmount:
+        return self._capacity
+
+    @capacity.setter
+    def capacity(self, capacity: ContainerAmount):
+        self._capacity = capacity
+        self.__update_capacity()
+
+    def __update_capacity(self):
+        self.visualization_manager.add_event(
+            ResourceSetCapacity(self.id, self.env.now, self.capacity)
+        )
+
+    def __update_utilization(self, _=None):
+        self.visualization_manager.add_event(
+            ResourceSetUtilization(
+                self.id, self.env.now, self.count))
 
     def request(
             self,
             priority: int = 0,
             preempt: bool = True) -> PriorityRequest:
         req = super().request(priority, preempt)
-        ResourceVisualUtil.set_utilization(self, self.count)
+        self.__update_utilization()
+        req.callbacks.append(self.__update_utilization)
         return req
 
     def release(self, request: PriorityRequest) -> Release:
         rel = super().release(request)
-        ResourceVisualUtil.set_utilization(self, self.count)
+        self.__update_utilization()
+        rel.callbacks.append(self.__update_utilization)
         return rel
 
 
@@ -210,7 +276,8 @@ class VisualContainer(VisualComponent, Container):
         0x0000FF is blue.
         If the whole image is white, tinting it will change the color of the
         image. If the image is black, tinting it will have no effect.
-        If no tint should be applied, set it to 0xFFFFFF.
+        If no tint should be applied, set it to 0xFFFFFF, which is the default
+        value.
     :raises TypeError: If the capacity is not a integer or float.
     :raises ValueError: If the capacity is not a positive integer or float.
     """
@@ -220,7 +287,7 @@ class VisualContainer(VisualComponent, Container):
         env: VisualEnvironment,
         id: str,
         visual: str,
-        tint: int,
+        tint: int = 0xFFFFFF,
         capacity: ContainerAmount = float("inf"),
         init: ContainerAmount = 0,
     ):
@@ -231,17 +298,38 @@ class VisualContainer(VisualComponent, Container):
         VisualComponent.__init__(
             self, env, id, ComponentType.CONTAINER, visual, tint)
         Container.__init__(self, env, capacity, init)
-        ContainerVisualUtil.set_capacity(self, capacity)
-        ContainerVisualUtil.set_level(self, self.level)
+        self.__update_capacity()
+        self.__update_level()
+
+    @property
+    def capacity(self) -> ContainerAmount:
+        return self._capacity
+
+    @capacity.setter
+    def capacity(self, capacity: ContainerAmount):
+        self._capacity = capacity
+        self.__update_capacity()
+
+    def __update_capacity(self):
+        self.visualization_manager.add_event(
+            ContainerSetCapacity(self.id, self.env.now, self.capacity)
+        )
+
+    def __update_level(self, _=None):
+        self.visualization_manager.add_event(
+            ContainerSetLevel(self.id, self.env.now, self.level)
+        )
 
     def put(self, amount: ContainerAmount) -> ContainerPut:
         put = super().put(amount)
-        ContainerVisualUtil.set_level(self, self.level)
+        self.__update_level()
+        put.callbacks.append(self.__update_level)
         return put
 
     def get(self, amount: ContainerAmount) -> ContainerGet:
         get = super().get(amount)
-        ContainerVisualUtil.set_level(self, self.level)
+        self.__update_level()
+        get.callbacks.append(self.__update_level)
         return get
 
 
@@ -260,7 +348,8 @@ class VisualStore(VisualComponent, Store):
         0x0000FF is blue.
         If the whole image is white, tinting it will change the color of the
         image. If the image is black, tinting it will have no effect.
-        If no tint should be applied, set it to 0xFFFFFF.
+        If no tint should be applied, set it to 0xFFFFFF, which is the default
+        value.
     :raises TypeError: If the capacity is not a integer or float.
     :raises ValueError: If the capacity is not a positive integer or float.
     """
@@ -270,7 +359,7 @@ class VisualStore(VisualComponent, Store):
         env: VisualEnvironment,
         id: str,
         visual: str,
-        tint: int,
+        tint: int = 0xFFFFFF,
         capacity: Union[float, int] = float("inf"),
     ):
         if not isinstance(capacity, (int, float)):
@@ -279,19 +368,41 @@ class VisualStore(VisualComponent, Store):
         if capacity <= 0:
             raise ValueError(
                 ErrorText.CAPACITY_MUST_BE_POSITIVE_INT_OR_FLOAT)
+        Store.__init__(self, env, capacity)
         VisualComponent.__init__(
             self, env, id, ComponentType.STORE, visual, tint)
-        Store.__init__(self, env, capacity)
-        StoreVisualUtil.set_capacity(self, capacity)
+        self.__update_capacity()
+        self.__update_content()
+
+    @property
+    def capacity(self) -> Union[float, int]:
+        return self._capacity
+
+    @capacity.setter
+    def capacity(self, capacity: Union[float, int]):
+        self._capacity = capacity
+        self.__update_capacity()
+
+    def __update_capacity(self):
+        self.visualization_manager.add_event(
+            StoreSetCapacity(self.id, self.env.now, self.capacity)
+        )
+
+    def __update_content(self, _=None):
+        self.visualization_manager.add_event(
+            StoreSetContent(self.id, self.env.now, self.items)
+        )
 
     def put(self, item) -> StorePut:
         put = super().put(item)
-        StoreVisualUtil.set_content(self, self.items)
+        self.__update_content()
+        put.callbacks.append(self.__update_content)
         return put
 
     def get(self) -> StoreGet:
         get = super().get()
-        StoreVisualUtil.set_content(self, self.items)
+        self.__update_content()
+        get.callbacks.append(self.__update_content)
         return get
 
 
@@ -311,7 +422,8 @@ class VisualFilterStore(VisualComponent, FilterStore):
         0x0000FF is blue.
         If the whole image is white, tinting it will change the color of the
         image. If the image is black, tinting it will have no effect.
-        If no tint should be applied, set it to 0xFFFFFF.
+        If no tint should be applied, set it to 0xFFFFFF, which is the default
+        value.
     :raises TypeError: If the capacity is not a integer or float.
     :raises ValueError: If the capacity is not a positive integer or float.
     """
@@ -322,23 +434,45 @@ class VisualFilterStore(VisualComponent, FilterStore):
             id: str,
             capacity: int,
             visual: str,
-            tint: int):
+            tint: int = 0xFFFFFF):
         if not isinstance(capacity, (int, float)):
             raise TypeError(ErrorText.CAPACITY_MUST_BE_POSITIVE_INT_OR_FLOAT)
         if capacity <= 0:
             raise ValueError(ErrorText.CAPACITY_MUST_BE_POSITIVE_INT_OR_FLOAT)
+        FilterStore.__init__(self, env, capacity)
         VisualComponent.__init__(
             self, env, id, ComponentType.STORE, visual, tint)
-        FilterStore.__init__(self, env, capacity)
-        StoreVisualUtil.set_capacity(self, capacity)
+        self.__update_capacity()
+        self.__update_content()
+
+    @property
+    def capacity(self) -> int:
+        return self._capacity
+
+    @capacity.setter
+    def capacity(self, capacity: int):
+        self._capacity = capacity
+        self.__update_capacity()
+
+    def __update_capacity(self):
+        self.visualization_manager.add_event(
+            StoreSetCapacity(self.id, self.env.now, self.capacity)
+        )
+
+    def __update_content(self, _=None):
+        self.visualization_manager.add_event(
+            StoreSetContent(self.id, self.env.now, self.items)
+        )
 
     def put(self, item) -> StorePut:
         put = super().put(item)
-        StoreVisualUtil.set_content(self, self.items)
+        self.__update_content()
+        put.callbacks.append(self.__update_content)
         return put
 
     def get(self, filter: Callable[[Any], bool]
             = lambda item: True) -> FilterStoreGet:
         get = super().get(filter)
-        StoreVisualUtil.set_content(self, self.items)
+        self.__update_content()
+        get.callbacks.append(self.__update_content)
         return get
